/*
 Non-Contact Blackout Detector
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/01/2012
 Last Modified: 8/01/2012
 
 A simple blackout detector using the ADC with a wire and 1 Megaohm resistor.
 If no AC interferance has been detected for 10 seconds (an attempt to detect interferance happens every 2 seconds), 
 it blinks the LED.
 You could link the output pin to another device, like a warning alarm, SMS sender, etc.
 
*/

// ATtiny25/45/85 Pin map
//                        +-\/-+
// Reset/Ain0 (D 5) PB5  1|o   |8  Vcc
//       Ain3 (D 3) PB3  2|    |7  PB2 (D 2) Ain1
//       Ain2 (D 4) PB4  3|    |6  PB1 (D 1) pwm1
//                  GND  4|    |5  PB0 (D 0) pwm0
//                        +----+
//
// Fuses: Set to factory defaults - Low 0x62, High 0xDF, Extended 0xFF

#define F_CPU 1000000 // 1 MHz clock

#define ledPin (1<<PB0) // LED
#define adcPin 3 // Analog input 3 (PB3)

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include "setup.c"

// Use this function for testing purposes when trying to find an interference source.
// LED will light when interference is detected.
void testInterference(void) {
	while(1) {
		
		int adcValue = 0;
		int x = 0;
		while (x < 10 && adcValue == 0) {
			_delay_ms(5);
			adcValue = analogRead(adcPin);
			x++;
		}
		
		if (adcValue != 0) { // AC interference detected
			PORTB |= ledPin;
			watchdogSleep(T250MS);
			PORTB &= ~ledPin;
			watchdogSleep(T250MS);
		}
		
		watchdogSleep(T250MS);
	}
}

int main(void) {
	
	setup();
	
	DDRB |= ledPin; // Set outputs
	PORTB |= ((1<<PB4) | (1<<PB2) | (1<<PB1)); // Turn on pull-up resistors on other ports to save power
	
	int timeOff = 0;
	
	// Uncomment the line below to test for interference sources. LED will light when interference is detected.
	//testInterference();
	
	while(1) {
		
		int adcValue = 0;
		int x = 0;
		while (x < 10 && adcValue == 0) { // Try to get an ADC reading of the AC interference 10 times in a row
			_delay_ms(5); // Wait a little while before ADC reading
			adcValue = analogRead(adcPin);
			x++;
		}
		
		if (adcValue == 0) { // No AC interference detected this run
			timeOff += 2; // Increment 2 seconds
		}
		else {
			timeOff = 0;
		}
		
		if (timeOff >= 10) { // No AC interference detected for more than 10 seconds
			PORTB |= ledPin;
			watchdogSleep(T250MS);
			PORTB &= ~ledPin;
			watchdogSleep(T250MS);
		}
		
		watchdogSleep(T2S);
	}
	
	return 0;
}

ISR(WDT_vect) { }
